@@ -4,22 +4,23 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-gTestfile = 'regress-355474-02.js';
+gTestfile = 'regress-354145-07.js';
 
-var BUGNUMBER = 355474;
-var summary = 'Iterating over XML with WAY_TOO_MUCH_GC';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 354145;
+var summary = 'Immutable XML';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-expect = '<a>text</a>';
-for each (var i in <><a>text</a></>)
-{
-    printStatus(i.toXMLString());
-    actual = i.toXMLString();
-}
+
+var a = <a><b/></a>
+try {
+  a.insertChildAfter(a.b[0], {toString: function() { throw 1; }});
+} catch (e) { }
+
+for each (var i in a) { }
 
 TEST(1, expect, actual);
 
