@@ -3,7 +3,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-316885-01.js';
+var gTestfile = 'regress-316885-03.js';
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 316885;
 var summary = 'Unrooted access in jsinterp.c';
@@ -13,22 +13,31 @@ var expect = '';
 printBugNumber(BUGNUMBER);
 printStatus (summary);
 
-var str_with_num = "0.1";
+var str = "test";
 
-var obj = {
-  elem getter: function() {
-    return str_with_num;
-  },
-  elem setter: function(value) {
-    gc();
+var lval = {
+  valueOf: function() {
+    return str+"1";
   }
-
 };
 
-expect = Number(str_with_num);
-actual = obj.elem++;
+var ONE = 1;
 
-gc();
+var rval = {
+  valueOf: function() {
+    // Make sure that the result of the previous lval.valueOf
+    // is not GC-rooted.
+    var tmp = "x"+lval;
+    if (typeof gc == "function")
+      gc();
+    for (var i = 0; i != 40000; ++i) {
+      tmp = 1e100 / ONE;
+    }
+    return str;
+  }
+};
+
+expect = (str+"1")+str;
+actual = lval+rval;
 
- 
 reportCompare(expect, actual, summary);
