@@ -3,16 +3,18 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-456477-01.js';
+var gTestfile = 'regress-360969-05.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 456477;
-var summary = 'Do not assert with JIT: (m != JSVAL_INT) || isInt32(*vp)" with (0/0)%(-1)';
+var BUGNUMBER = 360969;
+var summary = '2^17: local function';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
+var global = this;
 
 //-----------------------------------------------------------------------------
 test();
+
 //-----------------------------------------------------------------------------
 
 function test()
@@ -21,13 +23,22 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  jit(true);
+  var start = new Date();
+  var p;
+  var i;
+  var limit = 2 << 16;
 
-  for (var j = 0; j < 5; ++j) { var t = (0 / 0) % (-1); }
-
-  jit(false);
+  for (var i = 0; i < limit; i++)
+  {
+    eval('function pf' + i + '() {}');
+  }
 
   reportCompare(expect, actual, summary);
 
+  var stop = new Date();
+
+  print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
+
   exitFunc ('test');
 }
+
