@@ -4,7 +4,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 var gTestfile = 'regress-396326.js';
-
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 396326;
 var summary = 'Do not assert trying to disassemble get(var|arg) prop';
@@ -29,20 +28,10 @@ function test()
   }
   else
   {
-    function f1() { var v; return v.prop }; 
-    dis(f1);
+    function f4() { let local; return local.prop }; 
+    dis(f4);
     reportCompare(expect, actual, summary + 
-                  ': function f1() { var v; return v.prop };');
-
-    function f2(arg) { return arg.prop }; 
-    dis(f2);
-    reportCompare(expect, actual, summary + 
-                  ': function f2(arg) { return arg.prop };');
-
-    function f3() { return this.prop }; 
-    dis(f3);
-    reportCompare(expect, actual, summary + 
-                  ': function f3() { return this.prop };');
+                  ': function f4() { let local; return local.prop };');
   }
 
   exitFunc ('test');
