@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-470388-03.js';
+var gTestfile = 'regress-465460-05.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 470388;
-var summary = 'TM: Do not assert: !(fp->flags & JSFRAME_POP_BLOCKS)';
+var BUGNUMBER = 465460;
+var summary = 'TM: valueOf in a loop: do not assert';
 var actual = '';
 var expect = '';
 
@@ -20,10 +20,10 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
+
   jit(true);
 
-  for each (let x in [function(){}, {}, {}, function(){}, function(){}]) { ([<y/> for (y in x)]); }
+  for (var j = 0; j < 3; ++j) { 1 & Date; }
 
   jit(false);
 
